@@ -1,6 +1,6 @@
 from .core import Primitive, synchronized, PyThread, gated, TimerThread, Timeout
 from .dequeue import DEQueue
-from .task_queue import TaskQueue, Task, FunctionTask
+from .task_queue import TaskQueue, Task
 from .Scheduler import Scheduler
 from .Subprocess import Subprocess, ShellCommand
 from .RWLock import RWLock
@@ -9,7 +9,6 @@ from .promise import Promise
 from .processor import Processor
 from .flag import Flag
 from .gate import Gate
-from .LogFile import LogFile, LogStream
 from .producer import Producer
 from .escrow import Escrow
 
@@ -31,7 +30,7 @@ __a_ll__ = [
     'Version', '__version__', 'version_info',
     'Timeout',
     'Promise',
-    'Scheduler',
+    'Scheduler', 'Processor',
     'Gate', 'LogFile', 'LogStream',
     'Escrow', 'Producer'
 ]
