@@ -1,4 +1,4 @@
-from .core import Primitive, synchronized
+from .core import Primitive, synchronized, PyThread
 from .dequeue import DEQueue
 from .task_queue import Task, TaskQueue
 from .promise import Promise
@@ -13,46 +13,20 @@ class _WorkerTask(Task):
         self.Promise = promise
         
     def run(self):
-        try:
-            out = self.Processor._process(self.Item, self.Promise)
-        finally:
-            # unlink
-            self.Processor = None
-            self.Promise = None
-            self.Item = None
-            
-class _OutputIterator(object):
-    
-    def __init__(self, processor, queue):
-        self.Queue = queue
-        self.Processor = processor
+        out = self.Processor._process(self.Item, self.Promise)
         
-    def __next__(self):
-        try:    out, exc_info = self.Queue.pop()
-        except StopIteration:
-            self.Processor._remove_output_queue(self.Queue)
-            raise
-        if exc_info:
-            _, e, _ = exc_info
-            raise e
-        else:
-            return out
-            
-    def __del__(self):
-        self.Processor._remove_output_queue(self.Queue)
-        
-
 class Processor(Primitive):
     
-    def __init__(self, max_workers = None, queue_capacity = None, name=None, output = None, stagger=None, delegate=None,
+    Default = ""
+
+    def __init__(self, max_workers = None, queue_capacity = None, name=None, output = Default, stagger=None, delegate=None,
             put_timeout=None):
         Primitive.__init__(self, name=name)
-        self.Output = output
+        self.Output = DEQueue() if output is self.Default else output
         self.WorkerQueue = TaskQueue(max_workers, capacity=queue_capacity, stagger=stagger)
-        self.PutTimeout = put_timeout
         self.Delegate = delegate
-        self.OutputQueue = None
-        self.Closed = True
+        self.PutTimeout = put_timeout
+        self.Closed = False
         
     def hold(self):
         self.WorkerQueue.hold()
@@ -60,19 +34,35 @@ class Processor(Primitive):
     def release(self):
         self.WorkerQueue.release()
         
+    def open(self):
+        self.Output.open()
+        self.Closed = False
+
+    @synchronized
     def close(self):
         self.Closed = True
-        self.WorkerQueue.close()
-        
+        t = PyThread(target=self.wait_and_close_output)
+        t.start()
+    
+    def wait_and_close_output(self):
+        self.join()
+        self.Output.close()
+
+    @synchronized
     def put(self, item, timeout=-1):
+        if self.Closed:
+            raise RuntimeError("Processor is closed")
         if timeout == -1: timeout = self.PutTimeout
-        promise = Promise(item, name="initial")
+        promise = Promise()
         self.WorkerQueue.addTask(_WorkerTask(self, item, promise), timeout)
         return promise
-        
+
+    def get(self):
+        return self.Output.get()
+
     def join(self):
         return self.WorkerQueue.join()
-        
+
     def _process(self, item, promise):
         #print("%x: Processor._process: item: %s" % (id(self), item))
         try:    
@@ -82,23 +72,20 @@ class Processor(Primitive):
             if self.Delegate is not None:
                 self.Delegate.itemFailed(item, exc_type, exc_value, tb)
             promise.exception(exc_type, exc_value, tb)
-            if self.OutputQueue is not None:
-                self.OutputQueue.append((None, (exc_type, exc_value, tb)))
+            raise
         else:
             #print("%x: out: %s" % (id(self), out))
             if self.Delegate is not None:
                 self.Delegate.itemProcessed(item, out)
-            if out is not None and self.Output is not None:
-                next_promise = self.Output.put(out)
-                next_promise.Name = "secondary"
-                next_promise.chain(promise)     # fulfil this promise later, when the output processor has done with the item
-            else:
-                #print("%s: complete(%s)..." % (self, promise))
-                promise.complete(out)
-                if self.OutputQueue is not None:
-                    self.OutputQueue.append((out, None))
-                #print("%s: complete(%s) done" % (self, promise))
-            
+            if self.Output is not None and out is not None:
+                    next_promise = self.Output.put(out)
+                    if next_promise is not None:
+                        # fulfil this promise later, when the output processor has done with the item
+                        next_promise.Name = "secondary"
+                        next_promise.chain(promise)
+                        return
+            promise.complete(out)
+
     def process(self, item):
         # override me
         pass
@@ -120,19 +107,8 @@ class Processor(Primitive):
         
     def counts(self):
         return self.WorkerQueue.counts()
-        
-    @synchronized
+
     def __iter__(self):
-        if self.OutputQueue is not None:
-            raise RunTimeError("Can not open second iterator")
-        if self.Output is not None:
-            raise ValueError("Can not iterate over output of a processor sending its output to another processor")
-        self.OutputQueue = DEQueue()
-        return _OutputIterator(self, self.OutputQueue)
-        
-    @synchronized
-    def _remove_output_queue(self, queue):
-        if self.OutputQueue is queue:
-            self.OutputQueue = None
-        
-        
+        if self.Output is None:
+            raise RuntimeError("The processor does not have any output")
+        return iter(self.Output)
