@@ -24,6 +24,19 @@ class DebugLock(object):
 
 class Promise(Primitive):
     
+    class Callback(Primitive):
+        def __init__(self, oncomplete=None, onexception=None):
+            self.CompleteCB = oncomplete
+            self.ExceptionCB = onexception
+        
+        def promiseComplete(self, promise, result):
+            if self.CompleteCB is not None:
+                self.CompleteCB(promise, result)
+        
+        def promiseException(self, promise, exc_info):
+            if self.ExceptionCB is not None:
+                self.ExceptionCB(promise, *exc_info)
+
     def __init__(self, data=None, callbacks = [], name=None):
         Primitive.__init__(self, name=name)    #, lock=DebugLock())
         self.Data = data
@@ -35,34 +48,19 @@ class Promise(Primitive):
         self.OnDone = self.OnCancel = self.OnException = None
         self.Chained = []
         self.Name = name
-        
+
     @synchronized
-    def ondone(self, cb):
-        self.OnDone = cb
-        
-    @synchronized
-    def oncancel(self, cb):
-        self.OnCancel = cb
-        
-    @synchronized
-    def oncexception(self, cb):
-        self.OnException = cb
-        
-    @synchronized
-    def addCallback(self, cb):
-        if self.Complete and not self.Canceled:
-            cb(self)
-        self.Callbacks.append(cb)        
-        
-    @synchronized
-    def exception(self, exc_type, exc_value, exc_traceback):
-        self.ExceptionInfo = (exc_type, exc_value, exc_traceback)
-        if self.OnException is not None:
-            self.OnExceptipn(self)
-        for p in self.Chained:
-            p.exception(exc_type, exc_value, exc_traceback)
+    def add_callback(self, cb=None, oncomplete=None, onexception=None):
+        if not self.Cancelled:
+            if cb is None:
+                cb = self.Callback(oncomplete, onexception)
+            if self.ExceptionInfo and hasattr(cb, "promiseException"):
+                cb.promiseException(self, *self.ExceptionInfo)
+            elif self.Complete and hasattr(cb, "promiseComplete"):
+                cb.promiseComplete(self, self.Result)
+            else:
+                self.Callbacks.append(cb)
         self.wakeup()
-        self._cleanup()
 
     @synchronized
     def chain(self, *promises):
@@ -80,15 +78,23 @@ class Promise(Primitive):
             self.Chained += list(promises)
         return self
         
+    @synchronized
+    def cancel(self, cancel_chained=True):
+        self.Cancelled = True
+        self.Callbacks = []
+        if cancel_chained:
+            for p in self.Chained:
+                p.cancel(cancel_chained)
+        self.wakeup()
+        self._cleanup()
+
     @synchronized
     def complete(self, result=None):
         self.Result = result
         self.Complete = True
         if not self.Canceled:
-            if self.OnDone is not None:
-                self.OnDone(self.Result,)
             for cb in self.Callbacks:
-                if cb(self) == "stop":
+                if hasattr(cb, "promiseComplete") and cb.promiseComplete(self, self.Result) == "stop":
                     break
         for p in self.Chained:
             #print("%s: complete(%s) ..." % (self, p))
@@ -99,6 +105,22 @@ class Promise(Primitive):
     
     def is_complete(self):
         return self.Complete
+
+    def exception(self):
+        return self.ExceptionInfo
+        
+    def is_cancelled(self):
+        return self.Cancelled
+
+    @synchronized
+    def exception(self, exc_type, exc_value, exc_traceback):
+        self.ExceptionInfo = (exc_type, exc_value, exc_traceback)
+        if self.OnException is not None:
+            self.OnExceptipn(self)
+        for p in self.Chained:
+            p.exception(exc_type, exc_value, exc_traceback)
+        self.wakeup()
+        self._cleanup()
         
     @synchronized
     def wait(self, timeout=None):
@@ -117,37 +139,78 @@ class Promise(Primitive):
                 raise Timeout()
         finally:
             self._cleanup()
-            
-            
+
     def _cleanup(self):
         self.Chained = []
         self.Callbacks = []
-        self.OnCancel = self.OnException = self.OnDone = None
+        self.Callbacks = []
     
-class ORPromise(Primitive):
+    def __or__(self, other):
+        if isinstance(other, Promise):
+            return ORPromise([self, other])
+        else:
+            return other | self
+
+    def __and__(self, other):
+        if isinstance(other, Promise):
+            return ANDPromise([self, other])
+        else:
+            return other & self
+
+class ORPromise(Promise):
     
     def __init__(self, promises):
-        Primitive.__init__(self)
+        Promise.__init__(self)
         self.Fulfilled = None
+        self.Promises = promises
         for p in promises:
-            p.addCallback(self, promise_callback)
-
+            p.addCallback(self)
+            
     @synchronized
-    def promise_callback(self, promise):
+    def promiseComplete(self, promise, result):
         self.Fulfilled = promise
-        self.wakeup()
+        return self.complete(promise.Result)
     
     @synchronized
     def wait(self, timeout = None):
         while self.Fulfilled is None:
             self.sleep(timeout)
-        return sef.Fulfilled
+        return sef.Result
 
-class ANDPromise(Primitive):
+    def __or__(self, other):
+        if isinstance(other, ORPromise):
+            return ORPromise(other.Promises+self.Promises)
+        elif isinstance(other, (ANDPromise, Promise)):
+            return ORPromise([other]+self.Promises)
+        else:
+            raise ValueError("Can not apply '|' operation to", type(self), "and", type(other))
+
+    def __and__(self, other):
+        if isinstance(other, ANDPromise):
+            return ANDPromise(other.Promises + [self])
+        else:
+            return ANDPromise([self, other])
+
+class ANDPromise(Promise):
     
     def __init__(self, promises):
-        Primitive.__init__(self)
+        Promise.__init__(self)
         self.Promises = promises
         
     def wait(self, timeout=None):
-        return [p.wait(timeout) for p in self.Promises]
+        [p.wait(timeout) for p in self.Promises]
+        return self.complete([p.Result for p in self.Promises])
+
+    def __and__(self, other):
+        if isinstance(other, ANDPromise):
+            return ORPromise(other.Promises+self.Promises)
+        elif isinstance(other, (ORPromise, Promise)):
+            return ORPromise([other]+self.Promises)
+        else:
+            raise ValueError("Can not apply '&' operation to", type(self), "and", type(other))
+
+    def __or__(self, other):
+        if isinstance(other, ORPromise):
+            return ORPromise(other.Promises + [self])
+        else:
+            return ORPromise([self, other])
