@@ -39,6 +39,10 @@ class Task(Primitive):
         self._Private = _TaskPrivate()
         self._Private.After = None
         self._Private.Promise = None
+        
+    @property
+    def _promise(self):
+        return self._Private.Promise
     
     #def __call__(self):
     #    pass
@@ -376,7 +380,8 @@ class TaskQueue(PyThread):
         return len(self.Queue) == 0 and len(self.Threads) == 0
         
     isEmpty = is_empty
-                
+    
+    @synchronized
     def waitUntilEmpty(self):
         """
         Blocks until the queue is empty (no tasks are running and no tasks are waiting)
@@ -385,6 +390,8 @@ class TaskQueue(PyThread):
         if not self.isEmpty():
             while not self.sleep(function=self.isEmpty):
                 pass
+                
+    join = waitUntilEmpty
 
     def drain(self):
         """
